# -*- coding: utf-8 -*-
"""
@author: Christoph Lassner
"""
import numpy as np
import fertilized as fertilized

_SOIL_ = fertilized.Soil()

_REG_SOIL_ = fertilized.Soil('float', 'float', 'float',
                             result_type=fertilized.Result_Types.regression)
_N_THREADS = 1

class RegForestWrapper(object):
  def __init__(self, n_features=2,
                     max_depth=20,
                     features_per_node=2,
                     thresholds_per_node=10,
                     n_trees=10,
                     min_samples_per_leaf=2,
                     min_samples_per_split=5,
                     min_gain_threshold=0.,
                     allow_redraw=False,
                     random_seed=1,
                     entropy_name='shannon',
                     entropy_p1=2):
    self.n_features = n_features
    self.max_depth = max_depth
    self.features_per_node = features_per_node
    self.thresholds_per_node = thresholds_per_node
    self.n_trees = n_trees
    self.min_samples_per_leaf = min_samples_per_leaf
    self.min_samples_per_split = min_samples_per_split
    self.min_gain_threshold = min_gain_threshold
    self.allow_redraw = allow_redraw
    self.random_seed = random_seed
    self.entropy_name = entropy_name
    self.entropy_p1 = entropy_p1
    self.forest = _REG_SOIL_.StandardRegressionForest(self.n_features, # n features
                                                      self.max_depth,  # max depth
                                                      self.features_per_node, # n features per node
                                                      self.thresholds_per_node,  # n random thresholds per node
                                                      self.n_trees, # n trees
                                                      self.min_samples_per_leaf,   # min samples per leaf
                                                      self.min_samples_per_split,   # min samples per split
                                                      self.min_gain_threshold,  # min gain threshold
                                                      self.allow_redraw, # allow redraw,
                                                      self.random_seed,   # random seed
                                                      self.entropy_name, # entropy type
                                                      self.entropy_p1    # induced entropy param
                                                      )

  def fit(self, data, annot):
    self.forest.fit(np.ascontiguousarray(data.astype(_REG_SOIL_._inp_str)),
                    np.ascontiguousarray(annot.astype(_REG_SOIL_._ann_str)),
                    _N_THREADS)

  def predict(self, data):
    return np.mean(self.forest.predict(
    np.ascontiguousarray(data.astype(_SOIL_._inp_str)))[:, :, 0, :], axis=1)

  def get_params(self, deep=True):
    return {
    'n_features':self.n_features,
    'max_depth':self.max_depth,
    'features_per_node':self.features_per_node,
    'thresholds_per_node':self.thresholds_per_node,
    'n_trees':self.n_trees,
    'min_samples_per_leaf':self.min_samples_per_leaf,
    'min_samples_per_split':self.min_samples_per_split,
    'min_gain_threshold':self.min_gain_threshold,
    'allow_redraw':self.allow_redraw,
    'random_seed':self.random_seed,
    'entropy_name':self.entropy_name,
    'entropy_p1':self.entropy_p1
    }

  def set_params(self, n_features=None,
                       max_depth=None,
                       n_trees=None,
                       features_per_node=None,
                       thresholds_per_node=None,
                       min_samples_per_leaf=None,
                       min_samples_per_split=None,
                       min_gain_threshold=None,
                       allow_redraw=None,
                       random_seed=None,
                       entropy_name=None,
                       entropy_p1=None):
    self.n_features = n_features if not n_features is None else self.n_features
    self.max_depth = max_depth if not max_depth is None else self.max_depth
    self.features_per_node = features_per_node if not features_per_node is None else self.features_per_node
    self.thresholds_per_node = thresholds_per_node if not thresholds_per_node is None else self.thresholds_per_node
    self.n_trees = n_trees if not n_trees is None else self.n_trees
    self.min_samples_per_leaf = min_samples_per_leaf if not min_samples_per_leaf is None else self.min_samples_per_leaf
    self.min_samples_per_split = min_samples_per_split if not min_samples_per_split is None else self.min_samples_per_split
    self.min_gain_threshold = min_gain_threshold if not min_gain_threshold is None else self.min_gain_threshold
    self.allow_redraw = allow_redraw if not allow_redraw is None else self.allow_redraw
    self.random_seed = random_seed if not random_seed is None else self.random_seed
    self.entropy_name = entropy_name if not entropy_name is None else self.entropy_name
    self.entropy_p1 = entropy_p1 if not entropy_p1 is None else self.entropy_p1
    new_forest  = RegForestWrapper(self.n_features,
                                self.max_depth,
                                self.features_per_node,
                                self.thresholds_per_node,
                                self.n_trees,
                                self.min_samples_per_leaf,
                                self.min_samples_per_split,
                                self.min_gain_threshold,
                                self.allow_redraw,
                                self.random_seed,
                                self.entropy_name,
                                self.entropy_p1)
    self.forest = new_forest.forest
    return self

class ForestWrapper(object):
  def __init__(self, n_classes=2,
                     n_features=2,
                     max_depth=20,
                     features_per_node=2,
                     thresholds_per_node=10,
                     n_trees=10,
                     min_samples_per_leaf=2,
                     min_samples_per_split=5,
                     min_gain_threshold=0.,
                     allow_redraw=False,
                     random_seed=1,
                     entropy_name='shannon',
                     entropy_p1=2):
    self.n_classes = n_classes
    self.n_features = n_features
    self.max_depth = max_depth
    self.features_per_node = features_per_node
    self.thresholds_per_node = thresholds_per_node
    self.n_trees = n_trees
    self.min_samples_per_leaf = min_samples_per_leaf
    self.min_samples_per_split = min_samples_per_split
    self.min_gain_threshold = min_gain_threshold
    self.allow_redraw = allow_redraw
    self.random_seed = random_seed
    self.entropy_name = entropy_name
    self.entropy_p1 = entropy_p1
    self.forest = _SOIL_.StandardClassificationForest(self.n_classes,
                                                      self.n_features, # n features
                                                      self.max_depth,  # max depth
                                                      self.features_per_node, # n features per node
                                                      self.thresholds_per_node,  # n random thresholds per node
                                                      self.n_trees, # n trees
                                                      self.min_samples_per_leaf,   # min samples per leaf
                                                      self.min_samples_per_split,   # min samples per split
                                                      self.min_gain_threshold,  # min gain threshold
                                                      self.allow_redraw, # allow redraw,
                                                      self.random_seed,   # random seed
                                                      self.entropy_name, # entropy type
                                                      self.entropy_p1    # induced entropy param
                                                      )

  def fit(self, data, annot):
    self.forest.fit(data, annot, _N_THREADS)

  def predict(self, data):
    return np.argmax(self.forest.predict(data), axis=1)

  def get_params(self, deep=True):
    return {
    'n_classes':self.n_classes,
    'n_features':self.n_features,
    'max_depth':self.max_depth,
    'features_per_node':self.features_per_node,
    'thresholds_per_node':self.thresholds_per_node,
    'n_trees':self.n_trees,
    'min_samples_per_leaf':self.min_samples_per_leaf,
    'min_samples_per_split':self.min_samples_per_split,
    'min_gain_threshold':self.min_gain_threshold,
    'allow_redraw':self.allow_redraw,
    'random_seed':self.random_seed,
    'entropy_name':self.entropy_name,
    'entropy_p1':self.entropy_p1
    }

  def set_params(self, n_classes=None,
                       n_features=None,
                       max_depth=None,
                       n_trees=None,
                       features_per_node=None,
                       thresholds_per_node=None,
                       min_samples_per_leaf=None,
                       min_samples_per_split=None,
                       min_gain_threshold=None,
                       allow_redraw=None,
                       random_seed=None,
                       entropy_name=None,
                       entropy_p1=None):
    self.n_classes = n_classes if not n_classes is None else self.n_classes
    self.n_features = n_features if not n_features is None else self.n_features
    self.max_depth = max_depth if not max_depth is None else self.max_depth
    self.features_per_node = features_per_node if not features_per_node is None else self.features_per_node
    self.thresholds_per_node = thresholds_per_node if not thresholds_per_node is None else self.thresholds_per_node
    self.n_trees = n_trees if not n_trees is None else self.n_trees
    self.min_samples_per_leaf = min_samples_per_leaf if not min_samples_per_leaf is None else self.min_samples_per_leaf
    self.min_samples_per_split = min_samples_per_split if not min_samples_per_split is None else self.min_samples_per_split
    self.min_gain_threshold = min_gain_threshold if not min_gain_threshold is None else self.min_gain_threshold
    self.allow_redraw = allow_redraw if not allow_redraw is None else self.allow_redraw
    self.random_seed = random_seed if not random_seed is None else self.random_seed
    self.entropy_name = entropy_name if not entropy_name is None else self.entropy_name
    self.entropy_p1 = entropy_p1 if not entropy_p1 is None else self.entropy_p1
    new_forest  = ForestWrapper(self.n_classes,
                                self.n_features,
                                self.max_depth,
                                self.features_per_node,
                                self.thresholds_per_node,
                                self.n_trees,
                                self.min_samples_per_leaf,
                                self.min_samples_per_split,
                                self.min_gain_threshold,
                                self.allow_redraw,
                                self.random_seed,
                                self.entropy_name,
                                self.entropy_p1)

    self.forest = new_forest.forest
    return self
